<?php
// +----------------------------------------------------------------------
// | likeadmin快速开发前后端分离管理后台（PHP版）
// +----------------------------------------------------------------------
// | 欢迎阅读学习系统程序代码，建议反馈是我们前进的动力
// | 开源版本可自由商用，可去除界面版权logo
// | gitee下载：https://gitee.com/likeshop_gitee/likeadmin
// | github下载：https://github.com/likeshop-github/likeadmin
// | 访问官网：https://www.likeadmin.cn
// | likeadmin团队 版权所有 拥有最终解释权
// +----------------------------------------------------------------------
// | author: likeadminTeam
// +----------------------------------------------------------------------

namespace app\adminapi\lists\land;


use app\adminapi\lists\BaseAdminDataLists;
use app\common\model\land\LandPlantAction;
use app\common\lists\ListsSearchInterface;
use think\facade\Db;

/**
 * LandPlantAction列表
 * Class LandPlantActionLists
 * @package app\adminapi\listsland
 */
class LandPlantActionLists extends BaseAdminDataLists implements ListsSearchInterface
{


    /**
     * @notes 设置搜索条件
     * @return \string[][]
     * @author likeadmin
     * @date 2023/11/23 14:53
     */
    public function setSearch(): array
    {
        return [
            '=' => ['lpa.user_id', 'lpa.plant_id', 'lpa.type'],
        ];
    }

    /**
     * @notes 搜索条件
     * @author 段誉
     * @date 2023/2/24 16:08
     */
    public function querySearch()
    {
        $where = [];
        if (!empty($this->params['start_time'])) {
            $where[] = ['lpa.create_time', '>=', strtotime($this->params['start_time'])];
        }
        if (!empty($this->params['end_time'])) {
            $where[] = ['lpa.create_time', '<', strtotime($this->params['end_time'])];
        }
        return $where;
    }

    public function userSearch(): array
    {
        $userWhere['lpa.user_id'] = -1;
        // 超级管理员数据
        if ($this->adminInfo['root'] && !$this->adminInfo['user_id']) {
            unset($userWhere['lpa.user_id']);
        }
        // 普通用户数据
        if (!$this->adminInfo['root'] && $this->adminInfo['user_id']) {
            $userWhere['lpa.user_id'] = $this->adminInfo['user_id'];
        }
        return $userWhere;
    }

    /**
     * @notes 获取列表
     * @return array
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     * @author likeadmin
     * @date 2023/11/23 14:53
     */
    public function lists(): array
    {
        return Db::name('land_plant_action')->alias('lpa')
            ->where($this->searchWhere)->where($this->userSearch())->where($this->querySearch())
            ->leftJoin('user u','u.id = lpa.user_id')
            ->leftJoin('land_plant lp','lp.id = lpa.plant_id')
            ->field('lpa.*, lp.kind, lp.breed, u.account')
            ->limit($this->limitOffset, $this->limitLength)
            ->order(['lpa.id' => 'desc'])
            ->select()
            ->toArray();
    }


    /**
     * @notes 获取数量
     * @return int
     * @author likeadmin
     * @date 2023/11/23 14:53
     */
    public function count(): int
    {
        return Db::name('land_plant_action')->alias('lpa')
        ->where($this->searchWhere)->where($this->userSearch())->where($this->querySearch())
        ->leftJoin('user u','u.id = lpa.user_id')
        ->leftJoin('land_plant lp','lp.id = lpa.plant_id')
        ->count();
    }

}