<?php
// +----------------------------------------------------------------------
// | likeadmin快速开发前后端分离管理后台（PHP版）
// +----------------------------------------------------------------------
// | 欢迎阅读学习系统程序代码，建议反馈是我们前进的动力
// | 开源版本可自由商用，可去除界面版权logo
// | gitee下载：https://gitee.com/likeshop_gitee/likeadmin
// | github下载：https://github.com/likeshop-github/likeadmin
// | 访问官网：https://www.likeadmin.cn
// | likeadmin团队 版权所有 拥有最终解释权
// +----------------------------------------------------------------------
// | author: likeadminTeam
// +----------------------------------------------------------------------

namespace app\adminapi\lists\land;


use app\adminapi\lists\BaseAdminDataLists;
use app\common\model\land\LandPlant;
use app\common\lists\ListsSearchInterface;
use think\facade\Db;

/**
 * LandPlant列表
 * Class LandPlantLists
 * @package app\adminapi\listsland
 */
class LandPlantLists extends BaseAdminDataLists implements ListsSearchInterface
{


    /**
     * @notes 设置搜索条件
     * @return \string[][]
     * @author likeadmin
     * @date 2023/11/23 11:32
     */
    public function setSearch(): array
    {
        return [
            '%like%' => ['lp.kind', 'lp.breed'],
            '=' => ['lp.user_id', 'lp.land_id', 'lp.user', 'lp.status', 'lp.plant_date'],
        ];
    }

    /**
     * @notes 搜索条件
     * @author 段誉
     * @date 2023/2/24 16:08
     */
    public function querySearch()
    {
        $where = [];
        if (!empty($this->params['keyword'])) {
            $where[] = ['lp.kind|lp.breed', 'like', '%' . $this->params['keyword'] . '%'];
        }
        if (!empty($this->params['start_time'])) {
            $where[] = ['lp.plant_date', '>=', strtotime($this->params['start_time'])];
        }
        if (!empty($this->params['end_time'])) {
            $where[] = ['lp.plant_date', '<', strtotime($this->params['end_time'])];
        }
        return $where;
    }

    public function userSearch(): array
    {
        $userWhere['lp.user_id'] = -1;
        // 超级管理员数据
        if ($this->adminInfo['root'] && !$this->adminInfo['user_id']) {
            unset($userWhere['lp.user_id']);
        }
        // 普通用户数据
        if (!$this->adminInfo['root'] && $this->adminInfo['user_id']) {
            $userWhere['lp.user_id'] = $this->adminInfo['user_id'];
        }
        return $userWhere;
    }

    /**
     * @notes 获取列表
     * @return array
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     * @author likeadmin
     * @date 2023/11/23 11:32
     */
    public function lists(): array
    {
        return Db::name('land_plant')->alias('lp')
            ->where($this->searchWhere)->where($this->userSearch())->where($this->querySearch())
            ->leftJoin('user u','u.id = lp.user_id')
            ->leftJoin('land l','l.id = lp.land_id')
            ->field('lp.*, l.title, u.account')
            ->limit($this->limitOffset, $this->limitLength)
            ->order(['lp.id' => 'desc'])
            ->select()
            ->toArray();
    }


    /**
     * @notes 获取数量
     * @return int
     * @author likeadmin
     * @date 2023/11/23 11:32
     */
    public function count(): int
    {
        return Db::name('land_plant')->alias('lp')
        ->where($this->searchWhere)->where($this->userSearch())->where($this->querySearch())
        ->leftJoin('user u','u.id = lp.user_id')
        ->leftJoin('land l','l.id = lp.land_id')
        ->count();
    }

}