<?php
// +----------------------------------------------------------------------
// | likeadmin快速开发前后端分离管理后台（PHP版）
// +----------------------------------------------------------------------
// | 欢迎阅读学习系统程序代码，建议反馈是我们前进的动力
// | 开源版本可自由商用，可去除界面版权logo
// | gitee下载：https://gitee.com/likeshop_gitee/likeadmin
// | github下载：https://github.com/likeshop-github/likeadmin
// | 访问官网：https://www.likeadmin.cn
// | likeadmin团队 版权所有 拥有最终解释权
// +----------------------------------------------------------------------
// | author: likeadminTeam
// +----------------------------------------------------------------------

namespace app\adminapi\lists\land;


use app\adminapi\lists\BaseAdminDataLists;
use app\common\model\land\Product;
use app\common\lists\ListsSearchInterface;
use think\facade\Db;

/**
 * Product列表
 * Class ProductLists
 * @package app\adminapi\listsland
 */
class ProductLists extends BaseAdminDataLists implements ListsSearchInterface
{


    /**
     * @notes 设置搜索条件
     * @return \string[][]
     * @author likeadmin
     * @date 2023/11/25 16:16
     */
    public function setSearch(): array
    {
        return [
            '%like%' => ['p.name'],
            '=' => ['p.user_id', 'p.code', 'p.status'],
        ];
    }

    public function querySearch(): array
    {
        $queryWhere = [];
        if (!empty($this->params['land_id'])) {
            $queryWhere['l.id'] = $this->params['land_id'];
        }
        if (!empty($this->params['start_time'])) {
            $queryWhere[] = ['p.create_time', '>=', strtotime($this->params['start_time'])];
        }
        if (!empty($this->params['end_time'])) {
            $queryWhere[] = ['p.create_time', '<', strtotime($this->params['end_time'])];
        }
        return $queryWhere;
    }

    public function userSearch(): array
    {
        $userWhere['p.user_id'] = -1;
        // 超级管理员数据
        if ($this->adminInfo['root'] && !$this->adminInfo['user_id']) {
            unset($userWhere['p.user_id']);
        }
        // 普通用户数据
        if (!$this->adminInfo['root'] && $this->adminInfo['user_id']) {
            $userWhere['p.user_id'] = $this->adminInfo['user_id'];
        }
        return $userWhere;
    }

    /**
     * @notes 获取列表
     * @return array
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\DbException
     * @throws \think\db\exception\ModelNotFoundException
     * @author likeadmin
     * @date 2023/11/25 16:16
     */
    public function lists(): array
    {
            return Product::alias('p')
            ->where($this->searchWhere)->where($this->userSearch())->where($this->querySearch())
            ->with(['fenceHouse'])
            ->leftJoin('user u','u.id = p.user_id')
            ->field('p.*, u.account')
            ->limit($this->limitOffset, $this->limitLength)
            ->order(['p.id' => 'desc'])
            ->select()
            ->toArray();
    }


    /**
     * @notes 获取数量
     * @return int
     * @author likeadmin
     * @date 2023/11/25 16:16
     */
    public function count(): int
    {
        return Db::name('product')->alias('p')
            ->where($this->searchWhere)->where($this->userSearch())->where($this->querySearch())
            ->leftJoin('user u','u.id = p.user_id')
            ->count();
    }

}